unit DiiMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, OleCtrls, ExtCtrls, Buttons, DIIOCXLib_TLB;

type
  TBitSet=  set of 0..15;

  TForm1 = class (TForm)
    AutoTestBtn: TButton;
    CheckBox1: TCheckBox;
    CheckBox10: TCheckBox;
    CheckBox11: TCheckBox;
    CheckBox12: TCheckBox;
    CheckBox13: TCheckBox;
    CheckBox14: TCheckBox;
    CheckBox15: TCheckBox;
    CheckBox16: TCheckBox;
    CheckBox2: TCheckBox;
    CheckBox3: TCheckBox;
    CheckBox4: TCheckBox;
    CheckBox5: TCheckBox;
    CheckBox6: TCheckBox;
    CheckBox7: TCheckBox;
    CheckBox8: TCheckBox;
    CheckBox9: TCheckBox;
    DeviceBtn: TButton;
    Edit1: TEdit;
    Edit2: TEdit;
    Edit3: TEdit;
    Edit4: TEdit;
    Edit5: TEdit;
    Edit6: TEdit;
    Edit7: TEdit;
    Edit8: TEdit;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    GroupBox3: TGroupBox;
    GroupBox4: TGroupBox;
    GroupBox5: TGroupBox;
    Label1: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label2: TLabel;
    Label27: TLabel;
    Label28: TLabel;
    Label29: TLabel;
    Label3: TLabel;
    Label30: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    OKBtn: TButton;
    PageControl1: TPageControl;
    Shape1: TShape;
    tbClockRate: TEdit;
    tbElapsed: TEdit;
    tbOutputChannel: TEdit;
    tbOutputValue: TEdit;
    tbTimer: TEdit;
    tbTimerID: TEdit;
    Timer1: TTimer;
    tsAnalog: TTabSheet;
    tsDigital: TTabSheet;
    tsTimer: TTabSheet;
    UpDown1: TUpDown;
    UpDown2: TUpDown;
    ValueSlider: TTrackBar;
    DiiOcxCtrl: TDiiOcx;
    procedure AutoTestBtnClick(Sender: TObject);
    procedure DeviceBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure tbClockRateChange(Sender: TObject);
    procedure tbOutputChannelChange(Sender: TObject);
    procedure tbOutputValueChange(Sender: TObject);
    procedure tbTimerChange(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure ValueSliderChange(Sender: TObject);
  private
    FAutoTest: Boolean;
    FLocked: Boolean;
    FTestBit: Integer;
  protected
    function GetCheckBox(Index: Integer): TCheckBox;
    function GetEdit(Index: Integer): TEdit;
    function GetShape(Index: Integer): TShape;
    procedure SetValueSliderMax;
    procedure UpdateAnalog;
    procedure UpdateDigital;
    procedure UpdateTimer;
  public
    procedure Lock;
    procedure UnLock;
    property CheckBox[Index: Integer]: TCheckBox read GetCheckBox;
    property Edit[Index: Integer]: TEdit read GetEdit;
    property Locked: Boolean read FLocked write FLocked;
    property Shape[Index: Integer]: TShape read GetShape;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

{
************************************ TForm1 ************************************
}
procedure TForm1.AutoTestBtnClick(Sender: TObject);
begin
  FAutoTest :=not FAutoTest;
end;

procedure TForm1.DeviceBtnClick(Sender: TObject);
begin
  { disable timer events }
  Timer1.Enabled :=False;
  { ------------------------------------------------------
    try to select a Dii-Device, using SelectDevice
    ------------------------------------------------------ }
  if DiiOcxCtrl.SelectDevice then
    { fit slider with analog device resolution }
    SetValueSliderMax;
  { enable timer events again }
  Timer1.Enabled :=True;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  { ------------------------------------------------------
    try to select a Dii-Device, using SelectDevice
    ------------------------------------------------------ }
  if DiiOcxCtrl.SelectDevice then
  begin
    Timer1.Enabled := True;
    { fit slider with analog device resolution }
    SetValueSliderMax;
  end
  else
    Application.Terminate;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  Timer1.Enabled := False;
end;

function TForm1.GetCheckBox(Index: Integer): TCheckBox;
begin
  Result :=FindComponent(Format('CheckBox%d',[Index])) as TCheckBox;
end;

function TForm1.GetEdit(Index: Integer): TEdit;
begin
  Result :=FindComponent(Format('Edit%d', [Index])) as TEdit;
end;

function TForm1.GetShape(Index: Integer): TShape;
begin
  Result :=FindComponent(Format('Shape%d', [Index])) as TShape;
end;

procedure TForm1.Lock;
begin
  if FLocked then
    Abort;
  Locked :=True;
end;

procedure TForm1.OKBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TForm1.SetValueSliderMax;
begin
  { fit slider with analog device resolution }
  { set Max on the maximum value, the device-resolution supports }
  with ValueSlider do
  begin
    { -----------------------------------------------------------------------
      try to load the analog resolution the device supports, using Resolution
      ----------------------------------------------------------------------- }
    Max :=1 shl DiiOcxCtrl.Resolution -1;
    PageSize :=Max div 10;
  end;
end;

procedure TForm1.tbClockRateChange(Sender: TObject);
var
  aValue: Integer;
  aTimerID: Integer;
begin
  { output the new value to the selected Timer }
  aValue :=StrToIntDef(tbClockRate.Text, -1);
  { first check, if the value is valid }
  if aValue >-1 then
  begin
    { get the Timer-ID }
    aTimerID :=UpDown2.Position;
    { ------------------------------------------------------
      try to set a timer configuration, using SetTimerConfig
      ------------------------------------------------------ }
    DiiOcxCtrl.SetTimerConfig(aTimerID, aValue);
  end;
end;

procedure TForm1.tbOutputChannelChange(Sender: TObject);
begin
  { If the output channel changes, we just invoke
    the central "tbOutputValueChange" method, which
    performs the actual analog output }
  tbOutputValueChange (Sender);
end;

procedure TForm1.tbOutputValueChange(Sender: TObject);
var
  aValue: Integer;
  aChannel: Integer;
begin
  { output the new value to the selected channel: }
  { change Input into Integer}
  aValue :=StrToIntDef(tbOutputValue.Text, -1);
  { first check, if the value is valid }
  if aValue >-1 then
  begin
    aChannel :=UpDown1.Position;
    { ---------------------------------------------------------------
      output Data and check it was successful, using SetAnalogChannel
      --------------------------------------------------------------- }
    if not DiiOcxCtrl.SetAnalogChannel(aChannel, aValue) then
      tbOutputValue.Text := 'N/A';
  end;
end;

procedure TForm1.tbTimerChange(Sender: TObject);
var
  aValue: Integer;
  aTimerID: Integer;
begin
  { output the new value to the selected Timer }
  aValue :=StrToIntDef(tbClockRate.Text, -1);
  { first check, if the value is valid }
  if aValue >-1 then
  begin
    { get the Timer-ID }
    aTimerID :=UpDown2.Position;
    { -----------------------------------------------------------
      set the current value of a selected timer, using LoadTimer
      ----------------------------------------------------------- }
    DiiOcxCtrl.LoadTimer(aTimerID, aValue);
  end;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  { prevent recursion }
  Lock;
  {update analog I/O's }
  UpdateAnalog;
  {update digital I/O's }
  UpdateDigital;
  {update timer-chips }
  UpdateTimer;
  { enable timer events }
  UnLock;
end;

procedure TForm1.UnLock;
begin
  FLocked :=False;
end;

procedure TForm1.UpdateAnalog;
var
  i: Integer;
  aValue: Integer;
begin
  { Periodically, read the input values, and display
    the read data in the dialog box: }
  for i :=1 to 8 do
  begin
    { ----------------------------------------------------
      try to read a analog channel, using GetAnalogChannel
      ---------------------------------------------------- }
    if DiiOcxCtrl.GetAnalogChannel(i -1, aValue) then
      Edit[i].Text :=IntToStr(aValue)
    else
      Edit[i].Text :='N/A';
  end;
end;

procedure TForm1.UpdateDigital;
var
  aValue: TBitSet;
  aChannel: Integer;
  I: Integer;
begin
  { Our task here is to tell the OCX to set a specific
    channel, and to determine whether that operation
    was successful }
  for I :=1 to 16 do
    with CheckBox[I] do
    { ------------------------------------------------------
      try to write a bit, using SetDigitalByte
      ------------------------------------------------------ }
      if not DiiOcxCtrl.SetDigitalBit(I -1, Checked) then
         State :=cbGrayed;
  { Now we check the status of each input port,
    and display it accordingly: }
  for aChannel :=0 to 1 do
  begin
    { ------------------------------------------------------
      try to read channels 0 and 1, using GetDigitalByte
      ------------------------------------------------------ }
    if DiiOcxCtrl.GetDigitalByte (aChannel, SmallInt(aValue)) then
    begin
      for I :=1 to 8 do
        if I -1 in aValue then
          Shape[aChannel *8 +I].Brush.Color :=clLime
        else
          Shape[aChannel *8 +I].Brush.Color :=clGray;
    end
    else
      for I :=1 to 8 do
        Shape[aChannel *8 +I].Brush.Color :=clGray;
  end;
  {autotest function}
  if FAutoTest and (DiiOcxCtrl.DigitalChannels >0) then
  begin
    CheckBox[FTestBit +1].Checked :=False;
    Inc(FTestBit);
    if FTestBit =16 then
      FTestBit :=0;
    CheckBox[FTestBit +1].Checked :=True;
  end;
end;

procedure TForm1.UpdateTimer;
var
  aValue: Integer;
  aTimerID: Integer;
begin
  { get the Timer-ID }
  aTimerID :=UpDown2.Position;
  { ------------------------------------------------------------
    read the current value of the selected timer, using GetTimer
    ------------------------------------------------------------ }
  if DiiOcxCtrl.GetTimer(aTimerID, aValue) then
    tbElapsed.Text :=IntToStr(aValue)
  else
    tbElapsed.Text :='N/A';
end;

procedure TForm1.ValueSliderChange(Sender: TObject);
begin
  { When the user moves the slider, we setup the
    new output value. The slider moves from 0 to Max,
    which we interpret as a percentage }
  { we query the resolution of the selected device from the DII }
  if DiiOcxCtrl.Resolution >0 then
    tbOutputValue.Text :=IntToStr(ValueSlider.Position)
  else
    tbOutputValue.Text := 'N/A';
end;



end.
